package oracle.forms.jv;

import java.awt.*;
import java.text.DecimalFormat;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.image.*;
import oracle.forms.ui.VBean;
 

/**
 * A graphical gauge  -thermometer like
 * <P>
 * The gauge has the appearance of a thermometer gauge.
 * @author  Hafed Benteftifa created Jan 7, 2009
 * @version 0.1
 */
public class TGauge extends VBean  {
  private Point centre;
  private BufferedImage gaugeImage;
 //tick 
private int tickSize=12;
private Color tickColor=Color.black;
private String  tickFont="Arial";
  private String gaugeName;                      /* the gauge name */
  //current value
  private Color frameColor=Color.black;
  private int frameThickness=3;
  private Color frameBackgColor=Color.gray;
  private int valmin=0,valmax=100; /*marker for the min and max values*/
//disp 
private int dispSize=12;
private Color dispColor=Color.black;
private String dispFont="Arial";	 
//rings
private Color innerRingColor=Color.red;
private Color outerRingColor=Color.white;  
private double outerRing=80;
private double innerRing=60;
 private double meterRod;
  //gauge
   private Color gaugeBgColor=Color.white;
   private String gaugeUnit="Unit";
   private int unitSize=12;
   private int decimalFormat=0; //No decimals
     //dash
   private Color dashColor=Color.black;
   private int dashThickness=2;
   
private Color unitColor=Color.blue;
private String unitFont="Arial";
   private int size, tick;
    private int middle;
   private final static RenderingHints AALIAS = 
	new RenderingHints(RenderingHints.KEY_ANTIALIASING, 
			   RenderingHints.VALUE_ANTIALIAS_ON);
  
   private double val;                           /* the gauge value 0.0 - 1.0 */
  private Font f = new Font("Times Roman", Font.PLAIN, 14);
                                                

  
  /**
  * The constructor for Gauge. Creates a new Gauge
  * 
  */
  public TGauge() {
  	super();
  	String nativeLF = UIManager.getSystemLookAndFeelClassName();
    	try {
			UIManager.setLookAndFeel(nativeLF);	 
		} catch (Exception e) {
			e.printStackTrace();
		}
   }


/**
  * Set the   color property.
  *
  * @param     vColor - the Color value
  */
   public void setColor (Color vColor, int id){
   	switch(id){
   		case 152:
   			this.innerRingColor = vColor;//Set the inner ring color value
   			break;	
   		
   		case 151:
   			this.outerRingColor = vColor;//Set the outer ring color value 			
   			break;	
   				
   			case 12:
   			this.gaugeBgColor = vColor;//Set the gauge backgroundcolor value
   			break;			
   			
   			 case 133:
   			this.tickColor = vColor;//Set the tick color value
   			break;	
   			
   			 case 33:
   			this.unitColor = vColor;//Set the unit color value
   			break;		
   				
   			 case 92:
   			this.dispColor = vColor;//Set the disp color value
   			break;	
   			
   			case 51:
   			this.frameBackgColor = vColor;//Set the  frame background color
   			break;		
   				
   			case 50:
   			this.frameColor = vColor;//Set the  frame  color
   			break;
   			   			
   			case 170:
   			this.dashColor = vColor;//Set the  current value for the dash color
   			break;	
   	}
     this.repaint();
  }

/**
  * Set the   String property.
  *
  * @param     vString - the  String value
  */
   public void setStrProperty (String  vString, int id){
   	switch(id){
   		case 131:
   			this.tickFont = vString;//Set the tick font
   			break;	

   			case 90:
   			this.dispFont = vString;//Set the disp font
   			break;			
   			
   			 case 31:
   			this.unitFont = vString;//Set the unit font
   			break;	
   			
   			 case 30:
   			this.gaugeUnit = vString;//Set the gauge unit string
   			break;	
   			
   			case 11:
   			this.gaugeName = vString;//Set the gauge name string
   			break;  				
   								 
   	}
     this.repaint();
  }

/**
  * Set integer property.
  *
  * @param     vInteger - Integer value
  */
   public void setIntProperty (int  vInteger, int id){
   	switch(id){

   		case 132:
   			this.tickSize = vInteger;//Set the tick size
   			break;	
   		
		case 15:
   			this.decimalFormat = vInteger;//Set value in decimal format 1 or 0
   			break;
   				
   			case 91:
   			this.dispSize = vInteger;//Set the disp size
   			break;			
   			
   			 case 32:
   			this.unitSize = vInteger;//Set the unit size
   			break;			
   			  		
   			case 52:
   			this.frameThickness = vInteger;//Set the frame rect thickness
   			break;	
   				
   		   	case 13:
   			this.valmin = vInteger;//Set the min value
   			break;	
   		
   		case 14:
   			this.valmax = vInteger;//Set the max value
   			break;		
   				
   		case 171:
   			this.dashThickness = vInteger;//Set the dash thickness
   			break;							 				 
   	}
     this.repaint();
  }

 
  /**
  * Get the current name of the gauge.
  *
  */
  public String getGaugeName() {
     return this.gaugeName;
  }


// Paint the thermometer Gauge
/**
   *  This method paints the gauge  
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g: Graphics  context
   * </PRE>
   */
    public void paint (Graphics g) {
 	int width = getWidth();
	int height = getHeight();
 	//check that we have the correct surface for the gauge. Height should be bigger than width
         if ( width >  height){
        //	message("Height should be bigger than width!");
         	return;
         }
if ( width==0 || height == 0 ||  height <70) {
//	message("Size of the gauge is too small (should be more than 70 pixels)");
       return;
     }
     //check min and max values
     if ((valmax-valmin)==0)
	{
//		message("Error in setting current value");
		return;//bean has not selected max and min values
	}
     
outerRing=(height/(1.618*3.82));
double L=(2.5*height/(1.618*3.82));
double k=((height-2*8-.32*outerRing-L-outerRing)/4);//space at the top
double f = 3*k;//space at the bottom
  
centre=new Point(width/2, (int)(height-8-f-outerRing/2.0));
     //make sure we have enough space to draw the meter
    if (outerRing > width -20) {
outerRing = width -20 ;
     }   
     innerRing=outerRing -10;
     meterRod= 0.64*height/(1.618*3.82);  
      
     getParent().setBackground(gaugeBgColor);
	    Graphics2D g2d = (Graphics2D) g;
	     g2d.setBackground(getParent().getBackground());
	    g2d.addRenderingHints(AALIAS);  

 //... Create the thermometer  image if this is the first time,
        //    or if the size of the panel has changed. Image is buffered for subsequent paints.
        if (gaugeImage == null) {
            gaugeImage = (BufferedImage)(this.createImage(width, height));
            
            //... Get a graphics context from this image
            Graphics2D g2a = gaugeImage.createGraphics();
            g2a.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                 RenderingHints.VALUE_ANTIALIAS_ON); 
            drawGaugeFace(g2a, width, height, L);
        }
 	   g2d.drawImage(gaugeImage, null, 0, 0);
  	drawGaugeDynamic(g2d,width, height, L);
    }

  
/**
   *  This method draws the gauge face
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g2: Graphics2D context
   *L: height of the display rod
   * </PRE>
   */
   private void drawGaugeFace(Graphics2D  g2, int gWidth,int  gHeight, double L) {
        //  Draw the gauge face into a buffer.       
       double  gaugePercent=0;
         RoundRectangle2D rectFrame;   	
     
 //draw a rectangle for the gauge  
  g2.setColor(gaugeBgColor);     
 g2.fillRect(0,0,gWidth,gHeight);
     
//draw a rectangle enclosing the rest of the gauge
//save old thickness
java.awt.Stroke stroke = g2.getStroke();
g2.setStroke(new BasicStroke(frameThickness));
rectFrame =new RoundRectangle2D.Double(8, 8, gWidth-16, gHeight-16,10, 10);
 g2.setColor(frameBackgColor);
 g2.fill(rectFrame);
 g2.setColor(frameColor);
g2.draw(rectFrame);
g2.setStroke(stroke);//revert the line thickness

//locate center of gauge bottom circle
 Shape a = generateShape(g2,  L);
 g2.setColor(outerRingColor);
 g2.fill(a); 
 //inner shape thermometer
 double delta= (L/6.0);
 GradientPaint	redtowhite = new GradientPaint(centre.x -(int)((innerRing)/2),(int)(centre.y-(0.5*outerRing+delta*.5)),Color.white,centre.x -(int)((innerRing)/2), centre.y+(int)(innerRing/2),innerRingColor);
g2.setPaint(redtowhite);
 a = generateInnerShape(g2,  L);
g2.fill(a); 
 
  // Draw the tick marks and values for the thermometer.
 drawTicks(g2, L);
 drawPercent(g2, L); 
 	}

/**
   *  This method draws the outer shape
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g2: Graphics2D context
   *L: height of the display rod
   * </PRE>
   */
private Shape  generateShape(Graphics2D g, double L){
//bottom circle
Area area = new Area( new Ellipse2D.Double(centre.x -(int)((outerRing)/2),centre.y-(int)((outerRing)/2),  outerRing,outerRing));	
Area area1 = new Area( new RoundRectangle2D.Double(centre.x -(int)((meterRod)/2), centre.y-(int)(outerRing/2+L),meterRod ,L+10, 10, 10));
area.add(area1);	
//oval at the top outer
area1 = new Area( new Ellipse2D.Double(centre.x -(int)((meterRod)/2),centre.y-(int)(outerRing/2+L)-(int)((meterRod)/2),  meterRod ,meterRod ));	
area.add(area1);
 return area;
}

/**
   *  This method draws the inner shape
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g2: Graphics2D context
   *L: height of the display rod
   * </PRE>
   */
private Shape  generateInnerShape(Graphics2D g, double L){
//bottom circle
	Area area = new Area( new Ellipse2D.Double(centre.x -(int)((innerRing)/2),(int)(centre.y-(innerRing)/2),  innerRing,innerRing));	
	double delta= (L/6.0);
Area area1 = new Area( new Rectangle2D.Double(centre.x -(int)((innerRing)/4), (int)(centre.y-(0.5*outerRing+delta*.5)),(int)(innerRing/2) ,(int)(0.5*outerRing+delta*.5) ));
area.add(area1);	
 return area;
}



/**
   *  This method draws tick marks 
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g2: Graphics2D context
   *L: height of the display rod
   * </PRE>
   */
  private void drawTicks(Graphics2D g2, double L) {                            	
        //... percent parameter is the fraction (0.0 - 1.0) of the way
        g2.setColor(dashColor);   
       g2.setStroke(new BasicStroke(dashThickness));//set with the current thickness
  double delta= (L/6.0);          
  for (int sec = 0; sec <6; sec++) {
        g2.drawLine(centre.x -(int)((meterRod)/2), (int)(centre.y-0.5*outerRing-delta*.5-delta*sec), centre.x -(int)((meterRod)/2)+8, (int)(centre.y-0.5*outerRing-delta*.5-delta*sec));
  }  
    }

private void drawGaugeDynamic(Graphics2D g2, int gWidth, int gHeight, double L) {
    	double gaugePercent;
		/* put the gauge value on the screen */
	  //check whether we need decimals
	 DecimalFormat myFormat = new DecimalFormat("##0");
  if(decimalFormat!=0){
  	  myFormat = new DecimalFormat("##0.0");
  }	
  	// Center current value + unit horizontally
		//read actual properties
	f = new Font(dispFont, Font.PLAIN, dispSize);
     FontMetrics fm = getFontMetrics(f);
     g2.setFont(f);
     g2.setColor(dispColor);  
  	   gaugePercent=valmin+(val*(valmax-valmin));//double
     String buttonVal =  myFormat.format(gaugePercent) ;
  Rectangle2D rect = fm.getStringBounds(buttonVal, g2);
  int labelWidth  = (int)(rect.getWidth());//current value width
  
  	f = new Font(unitFont, Font.PLAIN, unitSize);
     fm = getFontMetrics(f);
     g2.setFont(f);
     g2.setColor(unitColor); 
  rect = fm.getStringBounds(gaugeUnit, g2);
  int labelWidth1  = (int)(rect.getWidth());//unit width is found
  //get starting point for drawing value and unit
  int labelStart = (gWidth  - labelWidth-labelWidth1)  / 2;
  
  //set draw property for current value
  f = new Font(dispFont, Font.PLAIN, dispSize);
       fm = getFontMetrics(f);
     g2.setFont(f);
     g2.setColor(dispColor); 
  g2.drawString(buttonVal, labelStart, gHeight -12);  // Draw the current value.
  
 //set draw property for unit
 	f = new Font(unitFont, Font.PLAIN, unitSize);
     fm = getFontMetrics(f);
     g2.setFont(f);
     g2.setColor(unitColor); 
  g2.drawString(gaugeUnit, labelStart+labelWidth, gHeight -12);  // Draw the string.
      //draw smooth movement from initial point to current value   
      double delta= L/6.0;       
       double intv=L*5.0/((valmax-valmin)*6.0); 
       int hh= (int)(centre.y-(0.5*outerRing+delta*.5+intv*(gaugePercent-valmin)));

GradientPaint redtowhite = new GradientPaint(centre.x -(int)((innerRing)/4),hh,innerRingColor,centre.x -(int)((innerRing)/4), hh+(int)(intv*(gaugePercent-valmin)),Color.white);
g2.setPaint(redtowhite);
 	g2.fillRect(  centre.x -(int)((innerRing)/4), hh,(int)(innerRing/2) ,(int)(intv*1.1*(gaugePercent-valmin)));    	     
  }



 /**  Alert dialog box
   * <P>
   * Parameters are as follows :-
   * <PRE>
   * <B>Message:</B>  String message to be displayed to the user
   * </PRE>
   */
private void message(String message) {
    JOptionPane.showMessageDialog(this, message, "Error", 
                                  JOptionPane.ERROR_MESSAGE);
  }
 

/**
   *  This method draws the scale values
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *g2: Graphics2D context
   *L: height of the display rod
   * </PRE>
   */
  private void drawPercent(Graphics2D g2,double L) {
     double gaugeTemp=((valmax-valmin)/5.0);
    
     double percent;
      int labelWidth,labelStart;
     Rectangle2D rect;
         g2.setColor(tickColor);   
        f = new Font(tickFont, Font.PLAIN, tickSize);
      FontMetrics fm = getFontMetrics(f);
     fm = getFontMetrics(f);
     g2.setFont(f);
     
    //check whether we need decimals
  DecimalFormat myFormat = new DecimalFormat("##0");
  if(decimalFormat!=0){
  	  myFormat = new DecimalFormat("##0.0");
  }
 	double delta= L/6.0;       
        for (int sec = 0; sec <6; sec++) {
        percent = valmin+gaugeTemp*sec;
       rect= fm.getStringBounds(myFormat.format(percent), g2);
   labelWidth  = (int)(rect.getWidth());
  labelStart = (int)((centre.x-8-meterRod  - labelWidth)  / 2);
    g2.drawString( myFormat.format(percent) , (int)(centre.x-5-.5*meterRod-labelWidth), (int)(centre.y-(0.5*outerRing+delta*.5+delta*sec)));  // Draw the string.
  }
    }
       
/**
  * Set the value of the Gauge in the range 0 to 1.0.
  *
  * @param     gaugeValue - the double value of the gauge
  */   
public void setGaugeValue(double val) {
	if ((valmax-valmin)==0)
	{
		message("Error in setting current value");
		return;//bean has not selected max and min values
	}
	val=(val-valmin)/(valmax-valmin);
	/* check and enforce the bounds on the gauge value */
 	if (val < 0) val = 0;
	if (val > 1) val = 1.0; 
	this.val = val;
	repaint();
    }
  
  
  /**
  * Get the value of the Gauge in the range val min to val max.
  *
  * @param     gaugeValue - the float value of the gauge
  */    
  public float getGaugeValue() {
	return (float)(val*(valmax-valmin)+valmin);
    }
   
}

