package oracle.forms.jv;

import oracle.forms.ui.*;
import oracle.forms.properties.ID;
import java.util.StringTokenizer;
import java.awt.*;
import javax.swing.*;
import oracle.forms.ui.CustomEvent;
import oracle.forms.handler.IHandler;

/**
 * A PJC which paints a thermometer on the screen 
 * <P>
 * Methods are defined to allow various properties of the gauge to be set
 * and read by Developer Forms through the use of the built-ins :-
 * <PRE>
 * <B>SET_CUSTOM_PROPERTY</B>
 *
 * and
 *
 * <B>GET_CUSTOM_PROPERTY</B>
 * </PRE>
 * Events can also be raised, and these will cause the
 * <B>WHEN-CUSTOM-ITEM-EVENT</B> trigger to be fired in the form.
 * <P>
 *For the latest version, please check www.degenio.com
 * @author  Hafed Benteftifa created Jan 7, 2009
 * @version 0.1
 */
public class TGaugeBean extends VBean {

   static IHandler mHandler; 
   	public final static ID INSTANCE      = ID.registerProperty("INSTANCE");
   	public final static ID ERROR      = ID.registerProperty("ERROR");
	//Gauge
  private static final ID GAUGE_VAL = ID.registerProperty("GAUGE_VAL");//10
    private static final ID GAUGE_NAME = ID.registerProperty("GAUGE_NAME");//11
   private static final ID GAUGE_BACKGROUND_COLOR = ID.registerProperty("GAUGE_BACKGROUND_COLOR"); //12 
    private static final ID VALMIN = ID.registerProperty("VALMIN");//13
private static final ID VALMAX = ID.registerProperty("VALMAX");//14
private static final ID DECIMAL_FORMAT = ID.registerProperty("DECIMAL_FORMAT");//15

    //unit
  private static final ID GAUGE_UNIT = ID.registerProperty("GAUGE_UNIT");//30
  private static final ID GAUGE_UNIT_FONT   = ID.registerProperty("GAUGE_UNIT_FONT");//31
private static final ID GAUGE_UNIT_SIZE  = ID.registerProperty("GAUGE_UNIT_SIZE");//32
private static final ID GAUGE_UNIT_COLOR  = ID.registerProperty("GAUGE_UNIT_COLOR");//33

//RECTANGLE-FRAME
   private static final ID FRAME_COLOR = ID.registerProperty("FRAME_COLOR");//50
   private static final ID FRAME_BACKGROUND_COLOR = ID.registerProperty("FRAME_BACKGROUND_COLOR");//51
   private static final ID FRAME_THICKNESS = ID.registerProperty("FRAME_THICKNESS");//52

//DISPLAY VALUE
private static final ID DISP_VALUE_FONT   = ID.registerProperty("DISP_VALUE_FONT");//90
private static final ID DISP_VALUE_SIZE  = ID.registerProperty("DISP_VALUE_SIZE");//91
private static final ID DISP_VALUE_COLOR  = ID.registerProperty("DISP_VALUE_COLOR");//92

//TICKS
private static final ID  TICK_VALUE_FONT = ID.registerProperty("TICK_VALUE_FONT");//131
private static final ID  TICK_VALUE_SIZE= ID.registerProperty("TICK_VALUE_SIZE");//132
private static final ID TICK_VALUE_COLOR = ID.registerProperty("TICK_VALUE_COLOR");//133
//rings-circle for gauge
private static final ID OUTER_RING_COLOR = ID.registerProperty("OUTER_RING_COLOR");//151
private static final ID INNER_RING_COLOR = ID.registerProperty("INNER_RING_COLOR");//152

//tick-dash
private static final ID  TICK_DASH_COLOR = ID.registerProperty("TICK_DASH_COLOR");//170
private static final ID TICK_DASH_THICKNESS = ID.registerProperty("TICK_DASH_THICKNESS");//171
 
  private String xGaugeName;
  private TGauge xGauge;
  private int propertyId;

  
  /**
  *  The constructor for a GaugeBean.
  * <P>
  *  If an Oracle Forms Bean Area is defined to have this class as
  *  its implementation, Forms will call this method to initially
  *  construct an instance of the bean at run time.
  */
  public TGaugeBean() {
    super();
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    xGauge = new TGauge();
      add("Center",xGauge);
         
        }

private void dispatchMessage( ID id)
  {
    try{
    CustomEvent ce = new CustomEvent(mHandler, id);
      dispatchCustomEvent(ce);  
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }    
  } 
  	
   public void init(IHandler handler)
{
super.init(handler);
mHandler = handler;
}	

private boolean setColor(int propId, Object value){
    	try{
      Color c=decodeColor((String)value);
      xGauge.setColor(c,propId);
      return true;
      	}catch (ClassCastException nx) {
            dispatchMessage(INSTANCE);
            return false;
        }
}
  /**
   *  This method sets the value of the requested property
   *  (identified by id) to the value in Object value. If
   *  this object does not support the specified property,
   *  the method returns FALSE. Otherwise, it returns TRUE.
   */
  public boolean setProperty (ID id, Object value)
  {
  	
  	int target_int;
  	boolean propResult=true;
  	//gauge
   if ( id.toString().equals("GAUGE_BACKGROUND_COLOR"))
    {
    propertyId=12;
    propResult=setColor(propertyId, value);
    return propResult;
    }
    
    if ( id.toString().equals("DECIMAL_FORMAT"))
    {
    propertyId=15;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      return true;
    }
    
     if ( id.toString().equals("GAUGE_UNIT"))
    {
      	propertyId=30;
      xGauge.setStrProperty((String)value,propertyId);
      return true;
    }
        if ( id.toString().equals("GAUGE_UNIT_FONT"))
    {
      	propertyId=31;
      xGauge.setStrProperty((String)value,propertyId);
      return true;
    }
    if ( id.toString().equals("GAUGE_UNIT_COLOR"))
    {
     propertyId=33;
        propResult=setColor(propertyId, value);
    return propResult;
    }
    if ( id.toString().equals("GAUGE_UNIT_SIZE"))
    {
    propertyId=32;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      return true;
    }
    
    //RINGS
    if ( id.toString().equals("INNER_RING_COLOR"))
    {
    	propertyId=152;
    propResult=setColor(propertyId, value);
    return propResult;
    }
     if ( id.toString().equals("OUTER_RING_COLOR"))
    {
    	propertyId=151;
    propResult=setColor(propertyId, value);
    return propResult;
    }

  	//disp
    if ( id.toString().equals("DISP_VALUE_FONT"))
    {
      	propertyId=90;
      xGauge.setStrProperty((String)value,propertyId);
      return true;
    }
    if ( id.toString().equals("DISP_VALUE_COLOR"))
    {
    propertyId=92;
    propResult=setColor(propertyId, value);
    return propResult;
    }
    if ( id.toString().equals("DISP_VALUE_SIZE"))
    {
       propertyId=91;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      return true;
    }


    if ( id == GAUGE_VAL )
    {
      double target_val=0;
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof String )
      {
      	try{
      		 target_val = Double.parseDouble((String)value.toString());
      	}catch (NumberFormatException nx) {
            dispatchMessage(ERROR);
            return false;
        }
      }
      else
      {
        /* don't do it */
        dispatchMessage(ERROR);
        System.out.println ("Illegal attempt to set GAUGE VALUE to " + value.getClass());
        return false;
      }
      xGauge.setGaugeValue(target_val  );
      return true;
    }
    
    if ( id == VALMIN )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof Integer )
      {
       propertyId=13;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set VAL Min to " + value.getClass());
        return false;
      }
      return true;
    }
    
     if ( id == VALMAX )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof Integer )
      {
         propertyId=14;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set VAL Max to " + value.getClass());
        return false;
      }
      return true;
    }
    
        if ( id.toString().equals("TICK_VALUE_FONT"))
    {
    	propertyId=131;
      xGauge.setStrProperty((String)value,propertyId);
      return true;
    }
    if ( id.toString().equals("TICK_VALUE_COLOR"))
    {
    propertyId=133;
    propResult=setColor(propertyId, value);
    return propResult;
    }
    if ( id.toString().equals("TICK_VALUE_SIZE"))
    {
       propertyId=132;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      return true;
    }
            
    if ( id == GAUGE_NAME )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof String )
      {
       	propertyId=11;
      xGauge.setStrProperty((String)value,propertyId);
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set Gauge NAME to " + value.getClass());
        return false;
      }
      return true;
    }
    
     if ( id.toString().equals("CURRENT_VALUE_COLOR"))
    {
    propertyId=110;
    propResult=setColor(propertyId, value);
    return propResult;
    }	    	 
    
    //dash tick
    if ( id == TICK_DASH_THICKNESS )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof Integer )
      {
      	    propertyId=171;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
           }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set current value for dash thickness to " + value.getClass());
        return false;
      }
      return true;
    }
    
      if ( id == TICK_DASH_COLOR )
    {  
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof String )
      {
      	propertyId=170;
    propResult=setColor(propertyId, value);
    return propResult;
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set color for tick dash color to " + value.getClass());
        return false;
      }    
    }  
    
     if ( id == FRAME_THICKNESS )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof Integer )
      {
       	    propertyId=52;
      xGauge.setIntProperty(((Integer)value).intValue(),propertyId);
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set frame thickness  to " + value.getClass());
        return false;
      }
      return true;
    }
    
    	         if ( id == FRAME_BACKGROUND_COLOR )
    {  
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof String )
      {
      	propertyId=51;
    propResult=setColor(propertyId, value);
    return propResult;
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set frame background color  to " + value.getClass());
        return false;
      }    
    }
    
           if ( id == FRAME_COLOR )
    {
      /* check for correct type passed to SET_CUSTOM_ITEM_PROPERTY */
      if ( value instanceof String )
      {
    propertyId=50;
    propResult=setColor(propertyId, value);
    return propResult;
      }
      else
      {
        /* don't do it */
        System.out.println ("Illegal attempt to set frame color  to " + value.getClass());
        return false;
      }    
    }
    return false;
  }


/**
   *  This method returns the color for an input string 
   *having the format: RXXXGXXXBXXX.
   * <P>
   * Parameter is as follows :-
   * <PRE>
   *rgbColor: Input string for the RGB color
   * </PRE>
   */
  public static Color decodeColor(String rgbColor) {
   int[] colorTokens={0,0,0};
   int ind=0;
    try {
      rgbColor = rgbColor.replace('R', '#');
      rgbColor = rgbColor.replace('G', '#');
      rgbColor = rgbColor.replace('B', '#');
     //String[] colorSep = rgbColor.split("\\s");//can't use split with current version of Jinitiator
      
     StringTokenizer colorSep= new StringTokenizer(rgbColor,"#");
     
     while (colorSep.hasMoreTokens()) {
    colorTokens[ind] =Integer.parseInt( colorSep.nextToken());
    ind++;
}
     checkColorRange(colorTokens);//check whether we are in the range (0-255)
         int red 	=  colorTokens[0];
     int green 	=   colorTokens[1];
     int blue 	= colorTokens[2];
        Color c  	= new Color(red, green, blue);
     return c;
   }catch(Exception e) { 
        System.out.println ("Illegal attempt to set Color value.");	      	         
    }
   return null;
}

/**
   *  This method sets the range for the color value to (0-255) 
      * <P>
   * Parameters -
   * <PRE>
   *colorValue: Input array of RGB colors
   * </PRE>
   */
private static void checkColorRange(int[] colorValue){
 	for (int i=0; i<3;i++){
 		if (colorValue[i]>255) colorValue[i]=255;
 		if (colorValue[i]<0) colorValue[i]=0;
 	}
 }

  /**
   *  This method returns the value of the requested property
   *  (identified by id) as a String in response to a request
   *  from Forms using GET_CUSTOM_PROPERTY.
   * <P>
   * Valid properties are as follows :-
   * <PRE>
   * <B>Gauge_Val</B>   the percentage value of the Gauge (0 - 100)
   * <B>Gauge_Name</B>  a String holding the name (and label) for the gauge
   * </PRE>
   */
  public Object getProperty(ID pid)
  {
    if ( pid ==GAUGE_NAME )
    {
      return ""+ xGauge.getGaugeName();
    }
    else if ( pid == GAUGE_VAL )
    {
      return "" + xGauge.getGaugeValue();
    }
    else
    {
      return super.getProperty(pid);
    }
  }

}

